unit ICButtonWhat;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the Button What, which drives the 10 or so buttons
	in the Main window that open the various preference windows.  There is not
	a lot of complexity here.  The basic function is to draw the buttons and
	respond to clicks on them.
	
	Drawing is slightly complicated by the silly grey backgrounds we implemented
	to surround the buttons.  These were necessary because the buttons looked
	really bad otherwise, but they make the code somewhat header.  The solution
	is, of course, to use the magic DeviceLoop routine to do all our drawing.
	
	Clicking is one of the layer breaks inside IC's design.  Technically, What
	modules have no cause to be importing ICWindows, but this module needs
	to call WindowOpen when the user clicks on a button.
*)

interface

	uses
		ICWindowGlobals;

	(* ***** Standard What Interface ***** *)
	
	(* See the comments in ICWindowGlobals. *)

	function WhatInitButton(dummy : integer) : OSStatus;
	function WhatOpenButton (wt: WindowType; item: integer): OSStatus;
	function WhatClickButton (wt: WindowType; item: integer; event: EventRecord): OSStatus;

implementation

	uses
		Traps,
		Icons,
		Dialogs,
		Fonts,

		ICResources,
		ICStringLists, 
		ICDialogs, 
		ICMiscSubs, 
		ICCommonSubs,
		ICWindowGlobals, 
		ICWindows;

	var
		gButtonUserItemUpdateUPP : UserItemUPP;
		gButtonDeviceLoopUPP : DeviceLoopDrawingUPP;

	(* ***** Utilities ***** *)

	procedure GetIconRect (window: WindowPtr; item: integer; var iconRect: Rect);
		(* This routine returns the position of the icon within the dialog
			item's bounding rectangle. The basic idea is to centre the icon,
			horizontally within the dialog item's rect, and position it
			just down from the top of the item.
		*)
		var
			dialogItemWidth: integer;
	begin
		GetDItemRect(window, item, iconRect);
		dialogItemWidth := iconRect.right - iconRect.left;
		iconRect.right := iconRect.left + 32;
		iconRect.bottom := iconRect.top + 32;
		OffsetRect(iconRect, (dialogItemWidth - 32) div 2, 8);
	end; (* GetIconRect *)

	(* ***** Drawing ***** *)

	var
		gButtonHighlighted: Boolean;
		(* gButtonHighlighted is a pseudo-parameter to ButtonDeviceLoopProc.  I can't
			add it as a direct parameter to ButtonDeviceLoopProc because ButtonDeviceLoopProc
			can be called by the system's DeviceLoop routine, which dictates which
			parameters go where.  It's true if the button should be drawn highlighted,
			false otherwise.
		*)

	procedure ButtonDeviceLoopProc (bitDepth: integer; deviceFlags: integer;
																targetDevice: GDHandle; item: longint);
		(* This routine is called by the system DeviceLoop routine repeatedly
			for each monitor that intersects the button's rectangle. The routine
			is expected to draw the button appropriately.  The system sets up
			the clip region such that extra drawing isn't a problem.
			
			The basic function of this routine is to draw the button, first
			by painting the background, then the frame, then the icon, and finally
			drawing the text.
		*)
		var
			window: WindowPtr;
			itemRect: Rect;
			colour: RGBColor;
			iconRect: Rect;
			borderRect: Rect;
			applicationFontInfo: FontInfo;
			width: integer;
	begin
		{$unused deviceFlags}
		{$unused targetDevice}
		GetPort(window);
		PenNormal;
		GetDItemRect(window, item, itemRect);
		
		(* Paint the background.  Note the four cases, ie whether the button is highlighted
			or not, and whether we have a colour display or not.  On a colour display
			we fill the background with a greyish colour, on a one bit display we
			fill it with either black or white.  The weird calls to lowrd are to
			turn 32 bit constants into 16 bit values.  The Pascal type for
			RGBColor defines the fields to be signed, which is completely wrong,
			so we have to compensate for it here.
		*)
		if gButtonHighlighted then begin
			if bitDepth = 1 then begin
				PaintRect(itemRect);
			end else begin
				colour.red := lowrd(16000);
				colour.green := lowrd(16000);
				colour.blue := lowrd(16000);
				RGBBackColor(colour);
				EraseRect(itemRect);
				BackColor(whiteColor);
			end; (* if *)
		end else begin
			if bitDepth = 1 then begin
				EraseRect(itemRect);
			end else begin

				// RGBColor is defined to have signed components,
				// but they are really signed.  When I compile with
				// CW Pro 1 and range checking enabled, it spits the dummy
				// when I try to assign 57015 to the signed component,
				// so I had to do the conversion myself.
				
				colour.red := -8521;
				colour.green := -8521;
				colour.blue := -8521;
				RGBBackColor(colour);
				EraseRect(itemRect);
				BackColor(whiteColor);
			end; (* if *)
		end; (* if *)
		
		(* Draw the frame. *)
		FrameRect(itemRect);
		
		(* Erase the icon area and draw the frame for the icon. *)
		GetIconRect(window, item, iconRect);
		borderRect := iconRect;
		InsetRect(borderRect, -4, -4);
		EraseRect(borderRect);
		FrameRect(borderRect);
		
		(* Now draw the icon. *)
		DrawIcon(rFirstButtonIcon + item - 1, iconRect, gButtonHighlighted);
		
		(* Finally draw the text label. *)
		TextFont(applFont);
		TextSize(9);
		GetFontInfo(applicationFontInfo);
		width := StringWidth(GetIndStr(rButtonLabels, item));
		MoveTo((itemRect.left + (itemRect.right - itemRect.left) div 2) - (width div 2),
					 itemRect.bottom - 2 - applicationFontInfo.descent);
		if gButtonHighlighted then begin
			TextMode(srcBic);
		end; (* if *)
		DrawString(GetIndStr(rButtonLabels, item));
		TextMode(srcOr);
	end; (* ButtonDeviceLoopProc *)

	procedure DrawButton (window: WindowPtr; item: integer; highlighted: Boolean);
		(* This routine draws the button associated with the What in
			the appropriate highlight state.
		*)
		var
			itemRect: Rect;
			itemRegion: RgnHandle;
	begin
		SetPort(window);

		(* Setup the pseudo-parameter to ButtonDeviceLoopProc. *)
		gButtonHighlighted := highlighted;
		
		(* Draw the button. If we don't have the DeviceLoop trap then we're obviously
			on a B&W machine, so we can just spoof out the ButtonDeviceLoopProc by
			calling it directly.  Otherwise we go ahead and call DeviceLoop.
		*)
		if TrapAvailable(_DeviceLoop) then begin
			GetDItemRect(window, item, itemRect);
			itemRegion := NewRgn;
			if itemRegion <> nil then begin
				RectRgn(itemRegion, itemRect);
				DeviceLoop(itemRegion, gButtonDeviceLoopUPP, item, 0);
				DisposeRgn(itemRegion);
			end; (* if *)
		end else begin
			ButtonDeviceLoopProc(1, 0, nil, item);
		end; (* if *)
	end; (* DrawButton *)

	procedure ButtonUserItemUpdate (window: WindowPtr; item: integer);
		(* This routine is called by the Dialog Manager to update the
			dialog item associated with the What.
		*)
	begin
		DrawButton(window, item, false);
	end; (* ButtonUserItemUpdate *)

	(* ***** Standard What Interface ***** *)

	function WhatInitButton(dummy : integer) : OSStatus;
	begin
		{$unused dummy}
		gButtonUserItemUpdateUPP := NewUserItemProc(@ButtonUserItemUpdate);
		gButtonDeviceLoopUPP := NewDeviceLoopDrawingProc(@ButtonDeviceLoopProc);
		WhatInitButton := noErr;
	end; (* WhatInitButton *)

	function WhatOpenButton (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals.
		
			The sole function of this routine is to install ButtonUserItemUpdate
			as the Dialog Manager user item update proc for the What.
		*)
	begin
		SetDItemHandle(gWindows[wt].window, item, gButtonUserItemUpdateUPP);
		WhatOpenButton := noErr;
	end; (* WhatOpenButton *)

	function WhatClickButton (wt: WindowType; item: integer; event: EventRecord): OSStatus;
		(* See the comments in ICWindowGlobals.
		
			This routine is basically one of your standard track-the-mouse-inside-
			a-rectangle routines. You've seen it a million times before.
		*)
		var
			window: WindowPtr;
			buttonHighlighted: Boolean;

		procedure ToggleHighlight;
		begin
			buttonHighlighted := not buttonHighlighted;
			DrawButton(window, item, buttonHighlighted);
		end; (* ToggleHighlight *)

		var
			err: OSStatus;
			mousePosition: Point;
			itemRect: Rect;
	begin
		{$unused event}
		window := gWindows[wt].window;
		GetDItemRect(window, item, itemRect);
		
		(* Track the mouse waiting for the button to come up. *)
		buttonHighlighted := false;
		repeat
			GetMouse(mousePosition);
			if PtInRect(mousePosition, itemRect) <> buttonHighlighted then begin
				ToggleHighlight;
			end; (* if *)
		until not StillDown;
		
		(* If the button was highlighted when we left the above loop,
			go ahead and try to open the window associated with this button.
		*)
		err := noErr;
		if buttonHighlighted then begin
			ToggleHighlight;
			err := WindowOpen(WindowType(ord(WT_Personal) + item - 1));
		end; (* if *)
		WhatClickButton := err;
	end; (* WhatClickButton *)

end. (* ICButtonWhat *)
